package org.greenstone.gsdl3.gs3build.extractor;

import java.io.FileReader;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentList;
import org.greenstone.gsdl3.gs3build.doctypes.GMLDocument;

public class GMLExtractor implements ExtractorInterface
{
  public static final String ACCUMULATE_MODE = "accumulate";

  /**
   *  An inner class to handle GML files
   */
  class GMLHandler extends DefaultHandler
  { List   files;
    String label;
    StringBuffer value;
    boolean inElement;
    boolean accumulate;

    GMLHandler()
    { super();
    
      this.file = null;
      this.label = null;
      this.value = null;
    }

    public void startElement(String URI, String localName, String qName, Attributes attributes)
    { if (localName.equals("FileName"))
      {	this.value = new StringBuffer();
      }
      else if (localName.equals("FileSet"))
      { this.files = new ArrayList();
      }
      else if (localName.equals("Description"))
      {
      }
      else if (localName.equals("Metadata"))
      { this.label = attributes.getValue("name");
        this.value = new StringBuffer();

	String mode = attributes.getValue("mode");
	this.accumulate = mode.equals(ACCUMULATE_MODE);	
      }
    }

    public void endElement(String URI, String localName, String qName)
    { if (localName.equals("FileName"))
      {	String file = this.value.toString();
        this.value = null;
	this.files.add(file);
      }
      else if (localName.equals("FileSet"))
      { // post the existing files item...
      }
      else if (localName.equals("Description"))
      {
      }
      else if (localName.equals("Metadata"))
      { GMLExtractor.postMetadata(this.files, this.label, this.value.toString());
        this.value = null;
	this.label = null;
      }
    }

    public void characters(char c[], int start, int length)
    { if (this.value != null)
      { String string = new String(c, start, length);
        this.value.append(string);
      }
    }
  }

  /**
   *  Construct of extractor
   */
  public GMLExtractor()
  { // Intentionally left blank
  }

  /**
   *  This extractor doesn't need to do any preparation/completion work,
   *  so this member function is empty.
   */
  public void configure(String outputDir)
  { // Intentionally left blank
  }

  public void configure(DocumentList documentList)
  { // Intentionally left blank
  }

  /**
   *  This extractor doesn't need to do any preparation/completion work,
   *  so this member function is empty.
   */
  public void startPass(int passNo)
  { // Intentionally left blank
  }

  /**
   *  Process the document - for a GML document, this results in the
   *  decoration of other files, for other documents, it does nothing.
   */
  public void extractDocument(DocumentID docID, DocumentInterface document)
  {	
    if (document.getDocumentType().equals(GMLDocument.GML_DOCUMENT_TYPE))
    { // Extract the content from the GML file
      try {
	XMLReader reader = XMLReaderFactory.createXMLReader();
	GMLHandler handler = new GMLHandler();
	reader.setContentHandler(handler);
	reader.setErrorHandler(handler);
	  
	// A GML document consists of one file only - get it from the 'default'
	// file group
	FileReader fileReader = new FileReader(document.getDocumentFiles().getFile(0).toString());
	reader.parse(new InputSource(fileReader));
      }
      catch (SAXException saxException)
      { // TODO: log error
      }
      catch (java.io.FileNotFoundException fileException)
      {
      }
      catch (java.io.IOException ioException)
      {
      }
      // for each document post it to the corresponding document
    }
  }

  protected static void postMetadata(List files, String value, String label)
  {
    
  }

  /**
   *  This extractor doesn't need to do any preparation/completion work,
   *  so this member function is empty.
   */
  public void endPass(int passNo)
  { // Intentionally left blank
  }

  /**
   *  This extractor is a simple, single-pass extractor
   *
   *  @see: org.greenstone.gsdl3.gs3build.extractor.ExtractorInterface:getNumberOfPasses
   */
  public int getNumberOfPasses()
  { return 1;
  }
}
